@@ -1,27 +1,19 @@
-# Standard lib imports
+import json
 import logging
 
-# Third party imports
-# None
-
-# Project level imports
-# None
-
-
 log = logging.getLogger(__name__)
 
 
 class Bucket(object):
-
     def __init__(self, connection):
         """
         Initialize a new instance
         """
         self.conn = connection
 
-    def create_bucket(self, bucket_name, vpool='', filesystem_enabled=False,
-                      head_type=None, namespace=None, is_stale_allowed=False,
-                      metadata=None):
+    def create(self, bucket_name, replication_group='', filesystem_enabled=False,
+               head_type=None, namespace=None, stale_allowed=False,
+               metadata=None, encryption_enabled=False):
         """
         Creates a bucket which could be used by users to create objects.
         The bucket is created in a storage pool associated with the specified
@@ -44,43 +36,55 @@ class Bucket(object):
         Example JSON result from the API:
 
         {
-            u'remote': None,
-            u'name': u'bucket-test1',
-            u'tags': [],
-            u'global': None,
-            u'vdc': None,
-            u'inactive': False,
-            u'id': u'namespace1.bucket-test1'
+            "name": "bucket5",
+            "id": "s3.bucket5",
+            "inactive": false,
+            "global": null,
+            "remote": null,
+            "vdc": null,
+            "tags": [],
+            "search_metadata":{
+              "metadata":[
+                 {
+                    "type": "User",
+                    "datatype":"integer",
+                    "name":"x-amz-meta-custom"
+                 }
+              ]
+            }
         }
 
         :param bucket_name: The bucket name
-        :param vpool:
-        :param filesystem_enabled:
-        :param head_type:
-        :param namespace: The namespace
-        :param is_stale_allowed:
-        :param search_metadata: Searchable metadata
+        :param replication_group: Replication group identifier
+        :param filesystem_enabled: Boolean indicating whether file-system is enabled for this bucket
+        :param head_type: Indicates the object head type that is allowed to access the bucket.
+        If the bucket has FS-Enabled, then the FS heads are implicitly allowed to access this bucket
+        :param namespace: Namespace associated with the user/tenant that is allowed to access the bucket
+        :param stale_allowed: Boolean indicating whether to allow stale data in bucket
+        :param metadata: Searchable tags assigned to objects created within the bucket.
+        Example: [{"name" : "x-amz-meta-custom", "type" : "User", "datatype" : "string"}, ...]
+        :param encryption_enabled: Boolean indicating whether is enabled for the bucket
         """
         payload = {
             "name": bucket_name,
-            "vpool": vpool,
+            "vpool": replication_group,
             "filesystem_enabled": filesystem_enabled,
-            "head_type": head_type,
-            "namespace": namespace,
-            "is_stale_allowed": is_stale_allowed,
-            "search_metadata": metadata
+            "is_stale_allowed": stale_allowed,
+            "is_encryption_enabled": encryption_enabled,
         }
 
-        log.info("Creating bucket '{0}': {1}".format(bucket_name, payload))
+        log.info("Creating bucket '{}'".format(bucket_name))
 
+        if namespace:
+            payload['namespace'] = namespace
         if head_type:
             payload['head_type'] = head_type
         if metadata:
             payload['search_metadata'] = metadata
 
-        return self.conn.post(url='object/bucket', json_payload=payload)
+        return self.conn.post('object/bucket', json_payload=payload)
 
-    def deactivate_bucket(self, bucket_name, namespace=None):
+    def delete(self, bucket_name, namespace=None):
         """
         Deletes the specified bucket.
 
@@ -94,20 +98,20 @@ class Bucket(object):
 
         Expect: HTTP/1.1 200 OK
 
-        :param bucket_name: The bucket name
-        :param namespace: The namespace
+        :param bucket_name: The bucket name to be deleted
+        :param namespace: Namespace associated. If not provided, then current user's namespace is used
         """
-        msg = "Deleting bucket '{0}'".format(bucket_name)
-        url = 'object/bucket/{0}/deactivate'.format(bucket_name)
+        msg = "Deleting bucket '{}'".format(bucket_name)
+        url = 'object/bucket/{}/deactivate'.format(bucket_name)
 
         if namespace:
-            url += '?namespace={0}'.format(namespace)
-            msg += " in namespace '{0}'".format(namespace)
+            url += '?namespace={}'.format(namespace)
+            msg += " in namespace '{}'".format(namespace)
 
         log.info(msg)
-        return self.conn.post(url=url)
+        return self.conn.post(url)
 
-    def get_buckets(self, namespace, marker='', limit=100):
+    def list(self, namespace, marker='', limit=100):
         """
         Gets the list of buckets for the specified namespace.
 
@@ -165,19 +169,18 @@ class Bucket(object):
             ]
         }
 
-        List of buckets associated with the namespace
-
         :param namespace: The namespace to query for buckets
         :param marker: Reference to last object returned
         :param limit: Number of objects requested in current fetch
         """
-        log.info("Getting all buckets in namespace '{0}'".format(namespace))
+        log.info("Getting all buckets in namespace '{}'".format(namespace))
 
-        return self.conn.get(
-            url='object/bucket?namespace={0}&marker={1}&limit={2}'.format(
-                namespace, marker, limit))
+        return self.conn.get('object/bucket?namespace={namespace}&marker={marker}&limit={limit}'.format(
+            namespace=namespace,
+            marker=marker,
+            limit=limit))
 
-    def set_bucket_retention(self, bucket_name, namespace, period=2592000):
+    def set_retention(self, bucket_name, namespace, period=2592000):
         """
         Updates the default retention setting for the specified bucket.
 
@@ -204,12 +207,10 @@ class Bucket(object):
         log.info("Setting retention for bucket '{0}' in namespace "
                  "'{1}': {2}".format(bucket_name, namespace, payload))
 
-        return self.conn.put(
-            url='object/bucket/{0}/retention'.format(bucket_name),
-            json_payload=payload
-        )
+        return self.conn.put('object/bucket/{}/retention'.format(bucket_name),
+                             json_payload=payload)
 
-    def get_bucket_retention(self, bucket_name, namespace=None):
+    def get_retention(self, bucket_name, namespace=None):
         """
         Gets the retention setting for the specified bucket.
 
@@ -227,17 +228,17 @@ class Bucket(object):
         :param namespace: Namespace associated. If it is null, then current
         user's Namespace is used.
         """
-        msg = "Getting retention for bucket '{0}'".format(bucket_name)
-        url = 'object/bucket/{0}/retention'.format(bucket_name)
+        url = 'object/bucket/{}/retention'.format(bucket_name)
+        msg = "Getting retention for bucket '{}'".format(bucket_name)
 
         if namespace:
-            url += '?namespace={0}'.format(namespace)
-            msg += " in namespace '{0}'".format(namespace)
+            url += '?namespace={}'.format(namespace)
+            msg += " in namespace '{}'".format(namespace)
 
         log.info(msg)
-        return self.conn.get(url=url)
+        return self.conn.get(url)
 
-    def get_bucket_info(self, bucket_name, namespace=None):
+    def get(self, bucket_name, namespace=None):
         """
         Gets bucket information for the specified bucket.
 
@@ -273,17 +274,17 @@ class Bucket(object):
         :param namespace: Namespace associated. If it is null, then current
         user's Namespace is used.
         """
-        msg = "Getting info for bucket '{0}'".format(bucket_name)
-        url = 'object/bucket/{0}/info'.format(bucket_name)
+        msg = "Getting info for bucket '{}'".format(bucket_name)
+        url = 'object/bucket/{}/info'.format(bucket_name)
 
         if namespace:
-            url += '?namespace={0}'.format(namespace)
-            msg += " in namespace '{0}'".format(namespace)
+            url += '?namespace={}'.format(namespace)
+            msg += " in namespace '{}'".format(namespace)
 
         log.info(msg)
-        return self.conn.get(url=url)
+        return self.conn.get(url)
 
-    def update_bucket_owner(self, bucket_name, new_owner, namespace=None):
+    def set_owner(self, bucket_name, new_owner, namespace=None):
         """
         Updates the owner for the specified bucket.
 
@@ -308,20 +309,26 @@ class Bucket(object):
         if namespace:
             payload['namespace'] = namespace
 
-        log.info("Updating owner for bucket '{0}': {1}".format(bucket_name,
-                                                               payload))
-        return self.conn.post(
-            url='object/bucket/{0}/owner'.format(bucket_name),
-            json_payload=payload
-        )
+        log.info("Updating owner for bucket '{}': {}".format(bucket_name, new_owner))
+        return self.conn.post('object/bucket/{}/owner'.format(bucket_name),
+                              json_payload=payload)
 
-    def update_bucket_is_stale_allowed(self, bucket_name, is_stale_allowed,
-                                       namespace=None):
+    def set_stale_allowed(self, bucket_name, stale_allowed, namespace=None):
         """
         Updates isStaleAllowed details for the specified bucket. If namespace
         does not exist in the request payload, the current user's namespace
         is used.
 
+        If you set this flag ON, and a temporary site outage occurs, objects that
+        you access in this bucket might have been updated at the failed site but
+        changes might not have been propagated to the site from which you are
+        accessing the object.Hence, you are prepared to accept that the objects
+        you read might not be up to date.
+
+        If the flag is turned OFF, data in the zone which has the temporary outage is not
+        available for access from other zones and object reads for data which has its
+        primary in the failed site will fail.
+
         Required role(s):
 
         SYSTEM_ADMIN
@@ -333,27 +340,26 @@ class Bucket(object):
 
         Expect: HTTP/1.1 200 OK
 
-        :param namespace: The namespace
-        :param is_stale_allowed: true/false
         :param bucket_name: Name of the bucket for which isStaleAllowed is to
         be updated
+        :param stale_allowed: true/false
+        :param namespace: Namespace associated with the user/tenant that is
+        allowed to access the bucket
         """
         payload = {
-            "is_stale_allowed": is_stale_allowed
+            "is_stale_allowed": stale_allowed
         }
 
         if namespace:
             payload['namespace'] = namespace
 
-        log.info("Updating 'isStaleAllowed' for bucket '{0}': {1}".format(
-                  bucket_name, payload))
+        log.info("Updating 'isStaleAllowed' for bucket '{}': {}".format(
+            bucket_name, stale_allowed))
 
-        return self.conn.post(
-            url='object/bucket/{0}/isstaleallowed'.format(bucket_name),
-            json_payload=payload
-        )
+        return self.conn.post('object/bucket/{}/isstaleallowed'.format(bucket_name),
+                              json_payload=payload)
 
-    def get_bucket_lock(self, bucket_name, namespace=None):
+    def get_lock(self, bucket_name, namespace=None):
         """
         Gets lock information for the specified bucket. The current user's
         namespace is used.
@@ -375,17 +381,17 @@ class Bucket(object):
         :param namespace: Name of the bucket for which lock information is to
         be retrieved
         """
-        msg = "Getting lock info for bucket '{0}'".format(bucket_name)
-        url = 'object/bucket/{0}/lock'.format(bucket_name)
+        msg = "Getting lock info for bucket '{}'".format(bucket_name)
+        url = 'object/bucket/{}/lock'.format(bucket_name)
 
         if namespace:
-            url += '?namespace={0}'.format(namespace)
-            msg += " in namespace '{0}'".format(namespace)
+            url += '?namespace={}'.format(namespace)
+            msg += " in namespace '{}'".format(namespace)
 
         log.info(msg)
-        return self.conn.get(url=url)
+        return self.conn.get(url)
 
-    def set_lock_bucket(self, bucket_name, is_locked='false', namespace=None):
+    def set_lock(self, bucket_name, is_locked=False, namespace=None):
         """
         Locks or unlocks the specified bucket. Current user's namespace
         is used.
@@ -402,27 +408,31 @@ class Bucket(object):
         Expect: HTTP/1.1 200 OK
 
         :param bucket_name: Name of the bucket which is to be locked/unlocked.
-        :param is_locked: Set to "true" for lock bucket and "false" for unlock
-        bucket.
+        :param is_locked: Set to True to lock the bucket and False to unlock the
+        bucket
         :param namespace: The namespace
         """
         payload = {}
-
         if namespace:
             payload['namespace'] = namespace
 
-        log.info("Setting lock to '{0}' for bucket '{1}': {2}".format(
-                  is_locked, bucket_name, payload))
+        log.info("Setting lock to '{}' for bucket '{}'".format(
+            is_locked, bucket_name))
 
         return self.conn.put(
-            url='object/bucket/{0}/lock/{1}'.format(bucket_name, is_locked),
-            json_payload=payload
-        )
+            'object/bucket/{}/lock/{}'.format(bucket_name, json.dumps(is_locked)),
+            json_payload=payload)
 
-    def update_bucket_quota(self, bucket_name, block_size, notification_size,
-                            namespace=None):
+    def set_quota(self, bucket_name, block_size, notification_size, namespace=None):
         """
-        Updates the quota for the specified bucket.
+        Updates the quota for the specified bucket. The payload specifies a limit
+        at which a notification will be raised in the event log and a limit at
+        which access will be blocked.
+
+        Both notification and block values must be supplied. If you do not want
+        to define one of them, you can set its value to -1. You cannot set both
+        values to -1 using this API. To set both notification and block values
+        to -1, use the delete quota API.
 
         Required role(s):
 
@@ -437,9 +447,12 @@ class Bucket(object):
 
         :param bucket_name: Name of the bucket for which the quota is to be
         updated
-        :param block_size:
-        :param notification_size:
-        :param namespace: The namespace
+        :param block_size: Block size in GB. Cannot be less than 1GB or use
+        decimal values. Can be set to -1 to indicate quota value not defined
+        :param notification_size: Notification size in GB. Cannot be less than
+        1GB or use decimal values. Can be set to -1 to indicate quota value not
+        defined
+        :param namespace: Namespace to which this bucket belongs
         """
         payload = {
             "blockSize": block_size,
@@ -449,18 +462,19 @@ class Bucket(object):
         if namespace:
             payload['namespace'] = namespace
 
-        log.info("Updating quota for bucket '{0}': {1}".format(bucket_name,
-                                                               payload))
+        log.info("Updating quota for bucket '{}'".format(bucket_name))
         return self.conn.put(
-            url='object/bucket/{0}/quota'.format(bucket_name),
-            json_payload=payload
-        )
+            'object/bucket/{0}/quota'.format(bucket_name),
+            json_payload=payload)
 
-    def get_bucket_quota(self, bucket_name, namespace=None):
+    def get_quota(self, bucket_name, namespace=None):
         """
         Gets the quota for the given bucket and namespace. The namespace with
         which the bucket is associated can be specified as a query parameter.
 
+        A value of -1 for the block or notification quota value indicates that
+        no quota has been defined.
+
         Required role(s):
 
         SYSTEM_ADMIN
@@ -481,17 +495,17 @@ class Bucket(object):
         :param namespace: Namespace with which bucket is associated. If it is
         null, the current user's namespace is used.
         """
-        msg = "Getting quota for bucket '{0}'".format(bucket_name)
-        url = 'object/bucket/{0}/quota'.format(bucket_name)
+        msg = "Getting quota for bucket '{}'".format(bucket_name)
+        url = 'object/bucket/{}/quota'.format(bucket_name)
 
         if namespace:
-            url += '?namespace={0}'.format(namespace)
-            msg += " in namespace '{0}'".format(namespace)
+            url += '?namespace={}'.format(namespace)
+            msg += " in namespace '{}'".format(namespace)
 
         log.info(msg)
-        return self.conn.get(url=url)
+        return self.conn.get(url)
 
-    def delete_bucket_quota(self, bucket_name, namespace=None):
+    def delete_quota(self, bucket_name, namespace=None):
         """
         Deletes the quota setting for the given bucket and namespace.
         The namespace with which the bucket is associated can be specified as
@@ -518,14 +532,14 @@ class Bucket(object):
         if namespace:
             params = {'namespace': namespace}
 
-        log.info("Deleting quota for bucket '{0}'".format(bucket_name))
+        log.info("Deleting quota for bucket '{}'".format(bucket_name))
 
         return self.conn.delete(
-            url='object/bucket/{0}/quota'.format(bucket_name),
+            url='object/bucket/{}/quota'.format(bucket_name),
             params=params
         )
 
-    def get_bucket_acl(self, bucket_name, namespace=None):
+    def get_acl(self, bucket_name, namespace=None):
         """
         Gets the ACL for the given bucket. Current user's namespace is used.
 
@@ -547,16 +561,14 @@ class Bucket(object):
         null, the current user's namespace is used.
         """
         params = None
-
         if namespace:
             params = {'namespace': namespace}
 
-        log.info("Getting ACL for bucket '{0}'".format(bucket_name))
+        log.info("Getting ACL for bucket '{}'".format(bucket_name))
 
         return self.conn.get(
-            url='object/bucket/{0}/acl'.format(bucket_name),
-            params=params
-        )
+            'object/bucket/{}/acl'.format(bucket_name),
+            params=params)
 
     def get_acl_permissions(self):
         """
@@ -612,7 +624,7 @@ class Bucket(object):
         }
         """
         log.info('Getting all ACLs')
-        return self.conn.get(url='object/bucket/acl/permissions')
+        return self.conn.get('object/bucket/acl/permissions')
 
     def get_acl_groups(self):
         """
@@ -652,4 +664,145 @@ class Bucket(object):
         }
         """
         log.info('Getting all ACL groups')
-        return self.conn.get(url='object/bucket/acl/groups')
+        return self.conn.get('object/bucket/acl/groups')
+
+    def set_acl(self):
+        raise NotImplementedError()
+
+    def get_metadata(self, bucket_name, head_type, namespace=None):
+        """
+        Fetch a page of head-specific metadata for the specified bucket
+
+        Required Role(s):
+
+        This call has no restrictions.
+
+        Example JSON result from the API:
+
+        {
+            'metadata': [
+                {'name': 'key1', 'value': 'value1'},
+                {'name': 'key2', 'value': 'value2'}
+            ],
+            'head_type': 'S3'
+        }
+
+        :param bucket_name: Name of the bucket for which the default group
+        is to be updated
+        :param head_type: the head-type of the metadata to be queried (HDFS,
+        S3, etc)
+        :param namespace: Namespace with which bucket is associated. If it is
+        null, the current user's namespace is used
+        """
+        params = {}
+        if namespace:
+            params['namespace'] = namespace
+        if head_type:
+            params['headType'] = head_type
+
+        log.info("Getting metadata for bucket '{}'".format(bucket_name))
+        return self.conn.get(
+            'object/bucket/{}/metadata'.format(bucket_name),
+            params=params)
+
+    def set_metadata(self, bucket_name, metadata_key, metadata_value, head_type,
+                     namespace=None):
+        """
+        Persist additional head metadata for the bucket
+
+        Required Role(s):
+
+        This call has no restrictions.
+
+        Example JSON result from the API:
+
+        :param bucket_name: Name of the bucket for which the metadata is to be added
+        :param metadata_key: Metadata key to be added
+        :param metadata_value: Metadata value to be added
+        :param head_type: the head-type of the metadata to be added (HDFS,
+        S3, etc)
+        :param namespace: Namespace with which bucket is associated. If it is
+        null, the current user's namespace is used
+        """
+        payload = {
+            "head_type": head_type,
+            "metadata": [
+                {
+                    "name": metadata_key,
+                    "value": metadata_value
+                }
+            ]
+        }
+        url = 'object/bucket/{}/metadata'.format(bucket_name)
+        if namespace:
+            url += '?namespace={}'.format(namespace)
+            payload['namespace'] = namespace
+
+        log.info("Adding metadata to bucket '{}'".format(bucket_name))
+        return self.conn.put(url, json_payload=payload)
+
+    def delete_metadata(self, bucket_name, head_type, namespace=None):
+        """
+        Delete a page of head metadata for the specified bucket
+
+        Required Role(s):
+
+        This call has no restrictions.
+
+        Example JSON result from the API:
+
+        :param bucket_name: name of the bucket for which the metadata is to be removed
+        :param head_type: the head-type of the metadata to be removed (HDFS, S3, etc)
+        :param namespace: Namespace with which bucket is associated. If it is
+        null, the current user's namespace is used
+        """
+        params = {'headType': head_type}
+        if namespace:
+            params['namespace'] = namespace
+
+        log.info("Deleting metadata for bucket '{}', head '{}'".format(bucket_name, head_type))
+        return self.conn.delete(
+            'object/bucket/{}/metadata'.format(bucket_name),
+            params=params)
+
+    def get_system_metadata_keys(self):
+        """
+        Lists the system metadata keys available.
+
+        Required Role(s):
+
+        This call has no restrictions.
+
+        Example JSON result from the API:
+
+        ...
+        """
+        log.info('Listing the system metadata keys available')
+        return self.conn.get('object/bucket/searchmetadata')
+
+    def disable_search_metadata(self, bucket_name, namespace=None):
+        """
+        Disables the metadata search functionality for a bucket.
+
+        Required Role(s):
+
+        This call has no restrictions.
+
+        There is no response body for this call
+
+        Expect: HTTP/1.1 200 OK
+
+        :param bucket_name: Bucket name for which metadata search mode will
+        be disabled
+        :param namespace: Namespace associated. If it is null, then current
+        user's namespace is used
+        """
+        params = None
+        if namespace:
+            params = {'namespace': namespace}
+
+        log.info("Disabling search metadata functionality for bucket '{}'".format(bucket_name))
+
+        return self.conn.delete(
+            'object/bucket/{}/searchmetadata'.format(bucket_name),
+            params=params)
